<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace TamerLib\Classes;

    use LogLib\Log;
    use Redis;
    use RedisException;
    use Symfony\Component\Process\Process;
    use TamerLib\Exceptions\NoAvailablePortException;
    use TamerLib\Exceptions\RedisServerException;

    class RedisServer
    {
        /**
         * @var string
         */
        private $cmd;

        /**
         * @var string
         */
        private $host;

        /**
         * @var int|null
         */
        private $port;

        /**
         * @var Process|null
         */
        private $server_process;

        /**
         * RedisServer constructor.
         *
         * @param string $cmd
         * @param string $host
         * @param int|null $port
         * @throws NoAvailablePortException
         */
        public function __construct(string $cmd='redis-server', string $host='127.0.0.1', ?int $port=null)
        {
            if (is_null($port))
            {
                $port = Utilities::getAvailablePort();
                Log::debug('net.nosial.tamerlib', 'Selected port ' . $port . '.');
            }

            $this->cmd = $cmd;
            $this->host = $host;
            $this->port = $port;
        }

        /**
         * Returns the port that the Redis server is listening on.
         *
         * @return int|null
         */
        public function getPort(): ?int
        {
            return $this->port;
        }

        /**
         * Determines if the Redis server is running.
         *
         * @return bool
         */
        public function isRunning(): bool
        {
            if(is_null($this->server_process))
            {
                return false;
            }

            return $this->server_process->isRunning();
        }

        /**
         * Starts the Redis server.
         *
         * @param int $timeout
         * @return bool
         * @throws RedisServerException
         * @throws RedisException
         */
        public function start(int $timeout=60): bool
        {
            if($this->isRunning())
            {
                return true;
            }

            Log::verbose('net.nosial.tamerlib', 'Starting Redis server on port ' . $this->port . '.');
            $this->server_process = new Process([$this->cmd, '--port', $this->port]);
            $this->server_process->start();

            // Use a redis client and ping the server until it responds.
            $redis_client = new Redis();
            $timeout_counter = 0;

            while(!$redis_client->isConnected())
            {
                if($timeout_counter >= $timeout)
                {
                    throw new RedisServerException('Redis server failed to start within ' . $timeout . ' seconds.');
                }

                try
                {
                    $redis_client->connect($this->host, $this->port);
                }
                catch (RedisException $e)
                {
                    // Do nothing.
                }
                finally
                {
                    sleep(1);
                    $timeout_counter++;
                }
            }
            
            Log::verbose('net.nosial.tamerlib', 'Redis server started.');
            return true;
        }

        /**
         * Stops the Redis server.
         *
         * @return bool
         */
        public function stop(): bool
        {
            if(!$this->isRunning())
            {
                return true;
            }

            $this->server_process->stop();
            Log::verbose('net.nosial.tamerlib', 'Redis server stopped.');
            return true;
        }

        /**
         * Terminates the Redis server.
         */
        public function __destruct()
        {
            $this->stop();
        }
    }